<?xml version='1.0'?>
<xsl:stylesheet xmlns:up="http://www.uportal.org" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
    <xsl:output method="xml"/>
    <xsl:param name="upgradeMajor">noUpgrade</xsl:param>
    <xsl:param name="upgradeMinor"></xsl:param>

    <xsl:template match="/">
        <statments>
            <xsl:apply-templates select="up:tables/up:table/up:foreign-key" mode="drop"/>
            <xsl:apply-templates select="up:tables/up:table" mode="drop"/>
            <xsl:apply-templates select="up:tables/up:table" mode="create"/>
            <xsl:apply-templates select="up:tables/up:table/up:index"/>
            <xsl:apply-templates select="up:tables/up:table/up:foreign-key" mode="create"/>
        </statments>
    </xsl:template>

    <xsl:template match="up:table" mode="drop">
        <statement type="tblDrop">
            <xsl:attribute name="name">
                <xsl:value-of select="up:name"/>
            </xsl:attribute>

            <xsl:text>DROP TABLE </xsl:text><xsl:value-of select="up:name"/>
        </statement>
    </xsl:template>

    <xsl:template match="up:table" mode="create">
        <xsl:choose>
            <!-- not upgrading or the upgrade version is older than the table version -->
            <xsl:when test="$upgradeMajor = 'noUpgrade' or @sinceMajor &gt; $upgradeMajor or (@sinceMinor &gt; $upgradeMinor and @sinceMajor &gt;= $upgradeMajor)">
                <statement type="tblCreate">
                    <xsl:attribute name="name">
                        <xsl:value-of select="up:name"/>
                    </xsl:attribute>

                    <xsl:text>CREATE TABLE </xsl:text><xsl:value-of select="up:name"/>
                    <xsl:text> (</xsl:text>
                    <xsl:apply-templates select="up:columns/up:column"/>
                    <xsl:apply-templates select="up:primary-key"/>
                    <xsl:apply-templates select="up:unique"/>
                    <xsl:text>)</xsl:text>
                </statement>
            </xsl:when>
            <!-- upgrading and table exists, check columns -->
            <xsl:otherwise>
                <xsl:for-each select="up:columns/up:column[@sinceMajor &gt; $upgradeMajor or (@sinceMinor &gt; $upgradeMinor and @sinceMajor &gt;= $upgradeMajor)]">
                    <statement type="tblAlter">
                        <xsl:attribute name="up:name">
                            <xsl:value-of select="../../up:name"/>
                        </xsl:attribute>

                        <xsl:text>ALTER TABLE </xsl:text>
                        <xsl:value-of select="../../up:name"/>
                        <xsl:text> ADD COLUMN </xsl:text>
                        <xsl:apply-templates select=".">
                            <xsl:with-param name="mode">alter</xsl:with-param>
                        </xsl:apply-templates>
                    </statement>
                </xsl:for-each>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="up:column">
        <xsl:param name="mode"/>

        <xsl:value-of select="up:name"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="up:type"/>
        <xsl:if test="up:param">
            <xsl:text>(</xsl:text>
            <xsl:value-of select="up:param"/>
            <xsl:text>)</xsl:text>
        </xsl:if>
        <xsl:if test="../../up:not-null = node() or ../../up:primary-key = node()">
            <xsl:text> NOT NULL</xsl:text>
        </xsl:if>
        <xsl:if test="$mode != 'alter' and (position() != last() or ../../up:primary-key or ../../up:unique)">
            <xsl:text>, </xsl:text>
        </xsl:if>
    </xsl:template>

    <xsl:template match="up:primary-key">
        <xsl:if test="position() = 1">
            <xsl:text>PRIMARY KEY (</xsl:text>
        </xsl:if>

        <xsl:value-of select="."/>

        <xsl:choose>
            <xsl:when test="position() = last()">
                <xsl:text>)</xsl:text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:text>, </xsl:text>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="up:unique">
        <xsl:if test="position() = 1">
            <xsl:text>,  UNIQUE (</xsl:text>
        </xsl:if>

        <xsl:value-of select="."/>

        <xsl:choose>
            <xsl:when test="position() = last()">
                <xsl:text>)</xsl:text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:text>, </xsl:text>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="up:index">
        <statement type="idxCreate">
            <xsl:attribute name="name">
                <xsl:value-of select="../up:name"/>
            </xsl:attribute>

            <xsl:text>CREATE INDEX </xsl:text>
            <xsl:value-of select="up:name"/>
            <xsl:text> ON </xsl:text>
            <xsl:value-of select="../up:name"/>
            <xsl:text> (</xsl:text>

            <xsl:for-each select="up:columns/up:column">
                <xsl:value-of select="."/>

                <xsl:if test="position() != last()">
                    <xsl:text>, </xsl:text>
                </xsl:if>
            </xsl:for-each>

            <xsl:text>)</xsl:text>
        </statement>
    </xsl:template>

    <xsl:template match="up:foreign-key" mode="create">
        <statement type="fkCreate">
            <xsl:attribute name="name">
                <xsl:value-of select="../up:name"/>
            </xsl:attribute>

            <xsl:text>ALTER TABLE </xsl:text>
            <xsl:value-of select="../up:name"/>
            <xsl:text> ADD CONSTRAINT </xsl:text>
            <xsl:value-of select="up:name"/>
            <xsl:text> FOREIGN KEY (</xsl:text>

            <xsl:for-each select="up:pair">
                <xsl:value-of select="up:column"/>

                <xsl:if test="position() != last()">
                    <xsl:text>, </xsl:text>
                </xsl:if>
            </xsl:for-each>

            <xsl:text>) REFERENCES </xsl:text>
            <xsl:value-of select="up:ref-table"/>
            <xsl:text> (</xsl:text>

            <xsl:for-each select="up:pair">
                <xsl:value-of select="up:ref-column"/>

                <xsl:if test="position() != last()">
                    <xsl:text>, </xsl:text>
                </xsl:if>
            </xsl:for-each>

            <xsl:text>)</xsl:text>
        </statement>
    </xsl:template>

    <xsl:template match="up:foreign-key" mode="drop">
        <statement type="fkDrop">
            <xsl:attribute name="name">
                <xsl:value-of select="../up:name"/>
            </xsl:attribute>

            <xsl:text>ALTER TABLE </xsl:text>
            <xsl:value-of select="../up:name"/>
            <xsl:text> DROP CONSTRAINT </xsl:text>
            <xsl:value-of select="up:name"/>
        </statement>
    </xsl:template>
</xsl:stylesheet>
